@#  GUBI - Gtk+ User Interface Builder
@#  Copyright (C) 1997	Tim Janik	<timj@psynet.net>
@#
@#  This program is free software; you can redistribute it and/or modify
@#  it under the terms of the GNU General Public License as published by
@#  the Free Software Foundation; either version 2 of the License, or
@#  (at your option) any later version.
@#
@#  This program is distributed in the hope that it will be useful,
@#  but WITHOUT ANY WARRANTY; without even the implied warranty of
@#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@#  GNU General Public License for more details.
@#
@#  You should have received a copy of the GNU General Public License
@#  along with this program; if not, write to the Free Software
@#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@#
@#  RCS_ID("$Id: main.tpl,v 1.5 1997/08/18 03:12:48 timj Exp $")
/* @fname@: reference main()
 * file created @Y@/@o@/@d@ @h@:@m@:@s@ by @builder_short@ (@fstate@)
*/
#include	"rcs.h"
RCS_ID("@_@")
RCS_KEY(Template, "$Template: main.tpl 1997 gubi $")

#define		__@fname_base@_@fname_postf@__

#include	"gbuild.h"
#include	"widgets.h"
#include	"ufunc.h"
#include	<signal.h>
#include	<unistd.h>



/* --- variables --- */
GHashTable	*gb_accelerator_hash_table=NULL;



/* --- prototypes --- */
static	void	gb_system_signal_handler	(int	sig_num);
static	void	gb_accel_hash_table_foreach	(gchar		*path,
						 gchar		*key_path,
						 gpointer	user_data);



/* --- main() --- */
int
main	(int	argc,
	 char	*argv[])
{
	static	guint	i;
	
#ifdef	DMALLOG
	dmg_init(NULL);
#endif	/*DMALLOG*/
	
	
	/* user defined argument parsing
	*/
	ufunc_parse_args(argc, argv);
	
	
	/* gtk/gdk initialization
	 */
	gtk_init(&argc, &argv);
	
	
	/* catch system signals
	*/
	gb_system_signal_handler(0);
	

	/* user defined program initialization
	*/
	ufunc_main_init(argc, argv);
	
	
	/* set up accelerator hash table
	*/
	gb_accelerator_hash_table=g_hash_table_new(g_string_hash, g_string_equal);
	gtk_accelerator_table_set_mod_mask(NULL,
					   GDK_SHIFT_MASK |
					   GDK_CONTROL_MASK |
					   GDK_MOD1_MASK);
	
	
	/* build windows
	*/
	for (i=0; i<gb_number_of_windows; i++) {
		if (gb_window_list[i] && GB_wCAST(window, gb_window_list[i])->auto_build)
			gb_window_build (gb_window_list[i],
					 gb_accelerator_hash_table,
					 gb_accelerator_hash_table);
	}
	
	
	/* connect signal handlers
	*/
	for (i=0; i<gb_number_of_windows; i++) {
		if (gb_window_list[i] && GB_wCAST(window, gb_window_list[i])->auto_build)
			gb_window_connect (gb_window_list[i]);
	}
	
	
	/* user defined widget initializations
	*/
	ufunc_widgets_init();
	
	
	/* show windows
	 */
	for (i=0; i<gb_number_of_windows; i++) {
		if (gb_window_list[i] && GB_wCAST(window, gb_window_list[i])->auto_build && gb_window_list[i]->flags&GTK_VISIBLE)
			gtk_widget_show(gb_window_list[i]->widget);
	}
	

	/* gtk's main loop
	 */
	gtk_main();
	
	
	/* save accelerator hash table
	*/
	g_hash_table_foreach(gb_accelerator_hash_table,
			    (GHFunc)gb_accel_hash_table_foreach,
			    NULL);
	g_hash_table_destroy(gb_accelerator_hash_table);
	
	
	/* user defined program shutdown
	*/
	ufunc_main_end();
	
	
	/* exit program
	*/
	gtk_exit(0);

	return 0;
}


void
gb_system_signal_handler	(int	sig_num)
{
	static	gboolean	in_call=FALSE;
	
	if (in_call) {
		fprintf(stderr, "\naborting on another signal: `%s'\n",
			g_strsignal(sig_num));
		fflush(stderr);
		gtk_exit(0);
	} else
		in_call=TRUE;
	
	signal(SIGINT,	gb_system_signal_handler);
	signal(SIGTRAP,	gb_system_signal_handler);
	signal(SIGABRT,	gb_system_signal_handler);
	signal(SIGBUS,	gb_system_signal_handler);
	signal(SIGSEGV,	gb_system_signal_handler);
	signal(SIGPIPE,	gb_system_signal_handler);
	/* signal(SIGTERM,	gb_system_signal_handler); */
	
	if (sig_num>0) {
		fprintf(stderr, "%s: (pid: %d) caught signal: `%s'\n",
			PRGNAME,
			getpid(),
			g_strsignal(sig_num));
		fflush(stderr);
		g_debug(PRGNAME);
	}
	in_call=FALSE;
}


void
gb_accel_hash_table_foreach	(gchar		*path,
				 gchar		*key_path,
				 gpointer	user_data)
{
	printf("left over: %s -> `%c' [%u]\n", path, key_path[0], key_path[1]);
	g_free(key_path);
}
